// Hardware access class for RC800 project
// Programmed by Jeroni Paul
/*
    HwAccess.h - layer to separate chip register details from rc800 program logic
    Copyright (C) 2007  Jeroni Paul

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

// In this version we use the DScaler driver version 4.1.
#ifndef __HWACCESS_H___
#define __HWACCESS_H___

// This structure holds the data to uniquely identify a card
struct TCard
{
	WORD dwVendorID;
	WORD dwDeviceID;
	int dwDeviceIndex;
	char *description;
} ;

class CHwAccess
{
public:

	// Class constructor and destructor
	CHwAccess();
	~CHwAccess();

	// Loads and starts DSDrv service
	bool LoadDriver();
	// Fills cards structure with all supported cards and returns 
	// number of cards detected
	int  FindCards(TCard *cards, int maxcards);
	// Selects a card to use
	bool UseCard(const TCard *card);

	// Functions to read and write GPIO pins. These provide a
	// generic interface independent of card make and model
	DWORD GetGPDATA();
	DWORD GetGPOE();

	// Returns all bits that are actually reading a physical pin
	// Bits 0 are outputs or being used for VIDEO/MPEG TS interface
	DWORD GetUsefulBits();

	// Enables an output to turn on IR chip
	void SetChipInit(DWORD wmask, DWORD wvalue);

	// Setting a mask makes GetGPDATA output the value for 1ms
	// to the IR chip. Used to ACK the read for cards that need that
	void SetChipACK(DWORD wmask, DWORD wvalue, DWORD ACKtime);

	// Performs ACK to the IR chip
	void ChipACK();

private:

	DWORD m_mask, m_initgpoe, m_initcard;
	DWORD m_wmask, m_wvalue;
	DWORD m_ACKtime;

	bool  m_maskFF, m_maskFF00, m_maskFF0000;

	TCard activeCard;
	DWORD m_subsystem;

	void  SetGPDATA(DWORD mask, DWORD value);
	void  SetGPOE(DWORD mask, DWORD value);
} ;

#endif