/*
    rc800.cpp - main program interface functions and control thread
    Copyright (C) 2007  Jeroni Paul

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include "rc800.h"
#include "HwAccess.h"
#include "settings.h"
#include "resource.h"

#include "GirderAPI.h"

// Capture card class
CHwAccess btCap;
#define MAXCARDS 16
TCard cards[MAXCARDS];
short numcards;

// Info
#define version "1.00"
#define name    "RC800 - BT8x8/CX2388x/SAA713x"
#define descr   "GPIO RC reader designed by Jeroni Paul"

// Global settings, Configuration data
DWORD datamask;
DWORD flagmask;
DWORD flagvalue;
bool noflag = true;
DWORD ACKmask, ACKvalue;
DWORD initmask, initvalue;
DWORD Interval = 100;
DWORD mask;
HINSTANCE hInstance;
WORD DeviceId;
int index;
DWORD pulseLen;
bool ACKalways;
bool enabled = false;

// Implementation for ARC2000
// Data for polling thread
TPluginCallBack PluginCallBack = NULL;
short status = 0; // 0: thread running, 1: signaled to stop, 2: exited

// Prototipes:
void IRThread( LPVOID lpParameter );

__declspec(dllexport) bool _stdcall HWInit()
{
	LPTSTR txt;
	size_t txt_len = sizeof(txt[0]) * 200;
	txt = (LPTSTR)malloc(txt_len);
	bool config;

	if(!btCap.LoadDriver())
	{
		LoadString(hInstance, IDS_NODRIVER, txt, txt_len);
		MessageBox(NULL, txt, "Error", MB_ICONERROR);
		free(txt);
		return false;
	}

	numcards = btCap.FindCards(cards, MAXCARDS);
	if(numcards == 0)
	{
		LoadString(hInstance, IDS_NOCARD, txt, txt_len);
		MessageBox(NULL, txt, "Error", MB_ICONERROR);
		free(txt);
		return false;
	}

	config = load_config();

	for(int i = 0; i < numcards; i++)
		if((cards[i].dwDeviceID == DeviceId) 
			&& (cards[i].dwDeviceIndex == index)) break;

	if(i == numcards)
		i = 0;
	
	if(!btCap.UseCard(&(cards[i])))
	{
		LoadString(hInstance, IDS_ERRORCARD, txt, txt_len);
		MessageBox(NULL, txt, "Error", MB_ICONERROR);
		free(txt);
		return false;
	}

	if(DeviceId == 0)
		datamask = btCap.GetUsefulBits();

	DeviceId = cards[i].dwDeviceID;
	index = cards[i].dwDeviceIndex;

	btCap.SetChipInit(initmask, initvalue);
	btCap.SetChipACK(ACKmask, ACKvalue, pulseLen);

	if(_beginthread(IRThread, 0, hInstance) == -1)
	{
		MessageBox(NULL, "Error creating Thread", "Error", MB_ICONERROR);
		status = 2;
		free(txt);
		return false;
	}
	
	free(txt);

	if(!config)
		RunConfig();

	return true;
}

__declspec(dllexport) bool _stdcall HWDeInit()
{
	if(!status)
	{
		//Stop the thread
		status = 1;
		while(status != 2) Sleep(80);
	}
	// Close config Window in case it is still open
	Terminate();
	return true;
}


__declspec(dllexport) bool _stdcall ConfigurationDialog()
{
	return RunConfig();
}

//************************* Remote Info ****************************
__declspec(dllexport) BOOL _stdcall GetRemoteInfo(PRemoteInfo RemoteInfo)
{
	RemoteInfo->Size = sizeof(TRemoteInfo);
	strcpy(RemoteInfo->Name, name);
	strcpy(RemoteInfo->Version, version);
	RemoteInfo->Flags = RF_CALLBACK | RF_CONFIGUREDIALOG;
	return true;
}

//************************* SetCallback ****************************
__declspec(dllexport) BOOL _stdcall SetCallback(TPluginCallBack CallBack)
{
	PluginCallBack = CallBack;
	enabled = (CallBack != NULL);
	return enabled;
}

/* Called by windows */
BOOL WINAPI DllMain( HANDLE hModule, 
                        DWORD fdwreason,  LPVOID lpReserved )
{
	switch(fdwreason) 
	{
    case DLL_PROCESS_ATTACH:
		hInstance = (HINSTANCE)hModule;
	//	break;
    //case DLL_THREAD_ATTACH:
    //break;
    //case DLL_THREAD_DETACH:
    //break;
    //case DLL_PROCESS_DETACH:
    //break;
    }
    return true;
}

// Polling thread
void IRThread( LPVOID lpParameter )
{
	int len;
	char buf[9];
	DWORD GPDATA;
	DWORD GPDATAprev;

	Sleep(1000);
	GPDATAprev = btCap.GetGPDATA() & mask;
	while(!status)
	{
		if(enabled)
		{
			// Poll for RC event
			GPDATA = btCap.GetGPDATA() & mask;
			if(ACKalways && ACKmask)
				btCap.ChipACK();
			if((noflag && (GPDATA != GPDATAprev)) 
				|| (flagmask && ((GPDATA & flagmask) == flagvalue)))
			{
				len = sprintf(buf, "%X", GPDATA & datamask);
				PluginCallBack(buf, len);
				GPDATAprev = GPDATA;
				if(!ACKalways && ACKmask) 
					btCap.ChipACK();
				Sleep(Interval);
			}
		}
		Sleep(50);
	}
	status = 2;
	_endthread();
}

// Implementation for Girder

#define devnum 2007
t_functions girder;

__declspec(dllexport) void _stdcall gir_version(char *data, int size)
{
	strncpy(data, version, size);
}

__declspec(dllexport) void _stdcall gir_name(char *data, int size)
{
	strncpy(data, name, size);
}

__declspec(dllexport) void _stdcall gir_description(char *data, int size)
{
	strncpy(data, descr, size);
}

__declspec(dllexport) int _stdcall gir_devicenum()
{
	return devnum;
}

__declspec(dllexport) int _stdcall gir_requested_api(int max_api)
{
	return 1;
}

__declspec(dllexport) int _stdcall gir_open(int gir_major_ver, int gir_minor_ver, 
               int gir_micro_ver, p_functions api_functions )
{
	girder = *api_functions;
	return HWInit() ? GIR_TRUE : GIR_FALSE;
}

__declspec(dllexport) int _stdcall gir_close()
{
	return HWDeInit() ? GIR_TRUE : GIR_FALSE;
}

__declspec(dllexport) void _stdcall gir_config()
{
	RunConfig();
}

bool _stdcall GirderCallBack(TKeyCode buf, DWORD size)
{
	girder.send_event(buf, NULL, 0, devnum);
	return true;
}

__declspec(dllexport) int _stdcall gir_start()
{
	PluginCallBack = GirderCallBack;
	enabled = true;
	return GIR_TRUE;
}

__declspec(dllexport) int _stdcall gir_stop()
{
	enabled = false;
	return GIR_TRUE;
}
