/*
    settings.cpp - functions for rc800 configuration dialog
    Copyright (C) 2007  Jeroni Paul

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <time.h>

#include "resource.h"
#include "settings.h"
#include "HwAccess.h"

// Access to bt8x8 card from object declared on bt8x8ir.cpp module
extern CHwAccess btCap;

extern TCard cards[];
extern short numcards;

// Access to settings from object bt8x8ir.cpp
extern DWORD datamask;
extern DWORD flagmask;
extern DWORD flagvalue;
extern bool noflag;
extern DWORD ACKmask, ACKvalue;
extern DWORD initmask, initvalue;
extern DWORD Interval;
extern DWORD mask;
extern HINSTANCE hInstance;
extern WORD DeviceId;
extern int index;
extern DWORD pulseLen;
extern bool ACKalways;
extern bool enabled;

// Global data for configuration dialog
HWND hWindow = NULL;
HWND hData, hPress, hValue, hInterval, hCards, hOvey, hACKalways, hInitMask, hInitValue, hACKmask, hACKvalue, hPulse;
HWND hRAW, hFiltered, hText, hFindPress, hSetOE, hACK, hOk, hCancel;
#define IDC_AUTO   100
#define IDC_ACK    101
#define IDC_SETOE  105
#define IDC_OK     102
#define IDC_CANCEL 103
#define IDC_OVEY   104
#define IDC_ACKALWAYS   106
bool classReg = false;
short status2, status3 = 2; // 0: thread running, 1: signaled to stop, 2: exited
bool refresh;
WNDPROC editwndproc, buttonwndproc, combowndproc;
LPTSTR txt;
const unsigned txt_len = sizeof(txt[0]) * 200;


// Prototipes

void LoadForm();
bool ok_click();
void auto_click();
void ack_click();
void setoe_click();
void cards_change();
void UpdateLoop( LPVOID lpParameter );
void AutoTh( LPVOID lpParameter );
bool DataAnalysis(DWORD list[], unsigned c, FILE *f);
DWORD FixRedundancy(DWORD mask, DWORD list[], unsigned c);


// Configuration functions
bool load_config()
{
	FILE *f;
	short ok = 0;
	unsigned flag;
	
	f = fopen("Rc800.ini", "rt");
	if(f != NULL)
	{
		// Load configurations from file
		ok = fscanf(f, "Device = %04X [%u]\nFlagMask = %X\nFlagValue = %X\nDataMask = %X\nInitmask = %X\nInitvalue = %X\nACKmask = %X\nACKvalue = %X\nOnlyOveyFlag = %u\nRepeatDelay = %lu\nPulseLength = %lu\nACKalways = %u\n",
			&DeviceId, &index, &flagmask, &flagvalue, &datamask, &initmask, &initvalue, &ACKmask, &ACKvalue, &flag, &Interval, &pulseLen, &ACKalways);
		noflag = (flag == 0);
		fclose(f);
	}

	if(ok != 13)
	{
		// Configuration file does not exist or is corrupt, default values
		mask = flagmask = ACKmask = initmask = ACKvalue = initvalue = 0;
		noflag = false;
		ACKalways = false;
		Interval = 300;
		pulseLen = 1;
		DeviceId = 0;
		return false;
	}
	mask = datamask | flagmask;
	return true;
}

bool save_config()
{
	FILE *f;
	short ok = -1;
	unsigned flag = 0;
	
	f = fopen("Rc800.ini", "wt");
	if(f != NULL)
	{
		if(!noflag) flag = 1;
		// Store configuration variables to file
		ok = fprintf(f, "Device = %04X [%u]\nFlagMask = %X\nFlagValue = %X\nDataMask = %X\nInitmask = %X\nInitvalue = %X\nACKmask = %X\nACKvalue = %X\nOnlyOveyFlag = %u\nRepeatDelay = %lu\nPulseLength = %lu\nACKalways = %u\n",
			DeviceId, index, flagmask, flagvalue, datamask, initmask, initvalue, ACKmask, ACKvalue, flag, Interval, pulseLen, ACKalways);
		fclose(f);
	}

	if(ok < 0)
	{
		// Error writing
		LoadString(hInstance, IDS_FILEERROR, txt, txt_len);
		MessageBox(NULL, txt, "Error", MB_ICONERROR);
		return false;
	}
	return true;
}

// Configuration window message process
LRESULT CALLBACK WindowProc(  HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch ( uMsg )
	{
	case WM_DESTROY:
		// Finish any thread that might be running
		if(!status3)
			status3 = 1;
		if(!status2)
			status2 = 1;
		Sleep(200);
		hWindow = NULL;
		break;

	case WM_COMMAND: // Buttons
		if(HIWORD(wParam) == BN_CLICKED)
			switch (LOWORD(wParam))
			{
			case IDC_OK:
				if(!ok_click()) break;

			case IDC_CANCEL:
				DestroyWindow(hwnd);
				break;

			case IDC_AUTO:
				auto_click();
				break;

			case IDC_ACK:
				ack_click();
				break;

			case IDC_SETOE:
				setoe_click();
			}
		else
			if(HIWORD(wParam) == CBN_SELCHANGE)
				cards_change();
		break;
	case WM_CHAR:
		if((char)LOWORD(wParam) == '\t')
			SetFocus(hCards);
	}
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

// Will allow only numbers 0 to 9 and letters A to F into hex textboxes
LRESULT CALLBACK EditProc(  HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char c;
	switch(uMsg)
	{
	case WM_CHAR:
		c = (char)toupper(LOWORD(wParam));
		if((c == '\b')
			|| ((c >= '0') && (c <= '9')) 
			|| ((c >= 'A') && (c <= 'F')))
		{	// Valid char typed
			refresh = true;
			break;
		}
		if(c == '\t')
		{
			SetFocus(GetNextDlgTabItem(hWindow, hwnd, FALSE));
			break;
		}
		return 0;
	}
	return CallWindowProc(editwndproc, hwnd, uMsg, wParam, lParam);
}

// Will set the focus when pressing tab key
LRESULT CALLBACK ButtonProc(  HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_CHAR:
		if((char)LOWORD(wParam) == '\t')
			SetFocus(GetNextDlgTabItem(hWindow, hwnd, FALSE));
	}
	return CallWindowProc(buttonwndproc, hwnd, uMsg, wParam, lParam);
}

// Will set the focus when pressing tab key
LRESULT CALLBACK ComboProc(  HWND hwnd,  UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_CHAR:
		if((char)LOWORD(wParam) == '\t')
			SetFocus(GetNextDlgTabItem(hWindow, hwnd, FALSE));
	}
	return CallWindowProc(combowndproc, hwnd, uMsg, wParam, lParam);
}

bool RunConfig()
{
	WNDCLASS WinC;
	HWND hLabel;
	char p[16];
	int i;

	if(hWindow)
	{  // If window is already open, set to foreground
		SetForegroundWindow(hWindow);
		return true;
	}

	if(!classReg)
	{
		WinC.style = 0;
		WinC.lpfnWndProc = (WNDPROC)WindowProc;
		WinC.cbClsExtra = 0;
		WinC.cbWndExtra = 0;
		WinC.hInstance = hInstance;
		WinC.hIcon = NULL;
		WinC.hCursor = LoadCursor(0, IDC_ARROW);
		WinC.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
		WinC.lpszClassName = "BT8x8IR";
		WinC.lpszMenuName = NULL;
		RegisterClass(&WinC);
		classReg = true;
	}

	txt = (LPTSTR) malloc(txt_len);

	LoadString(hInstance, IDS_TITLE, txt, txt_len);
	hWindow = CreateWindow("BT8x8IR", txt,
				WS_OVERLAPPED | WS_SYSMENU, CW_USEDEFAULT, CW_USEDEFAULT,
                400, 450, NULL, NULL, hInstance, NULL);

	if(!hWindow)
	{
		MessageBox(NULL, "Cannot show dialog", "Error", MB_ICONERROR);
		return false;
	}

	// Card selector:
	LoadString(hInstance, IDS_CARDS, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 14,
            60, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hCards = CreateWindowEx(WS_EX_CLIENTEDGE, "ComboBox", NULL,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP | CBS_DROPDOWNLIST,
			70, 12, 120, 300, hWindow, NULL, hInstance, NULL);

	SendMessage(hCards, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	combowndproc = (WNDPROC)SetWindowLong(hCards, GWL_WNDPROC, (LONG)ComboProc);


	// Polling interval:
	LoadString(hInstance, IDS_POLLING, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 190, 14,
            135, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hInterval = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_NUMBER | WS_TABSTOP | SS_RIGHT,
			335, 12, 40, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hInterval, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hInterval, EM_LIMITTEXT , 4, 0);
	editwndproc = (WNDPROC)SetWindowLong(hInterval, GWL_WNDPROC, (LONG)EditProc);

	// Explanation
	LoadString(hInstance, IDS_INST1, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE, 10, 40,
            370, 35, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	// Press mask
	LoadString(hInstance, IDS_PRESSMASK, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 78,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hPress = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			110, 77, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hPress, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hPress, EM_LIMITTEXT, 8, 0);
	SetWindowLong(hPress, GWL_WNDPROC, (LONG)EditProc);

	// Press value
	LoadString(hInstance, IDS_VALUE, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 190, 78,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hValue = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			300, 77, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hValue, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hValue, EM_LIMITTEXT , 8, 0);
	SetWindowLong(hValue, GWL_WNDPROC, (LONG)EditProc);

	// Data mask
	LoadString(hInstance, IDS_DATAMASK, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 100,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hData = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			110, 99, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hData, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hData, EM_LIMITTEXT , 8, 0);
	SetWindowLong(hData, GWL_WNDPROC, (LONG)EditProc);

	// Checkbox - ovey press flag
	LoadString(hInstance, IDS_OVEY, txt, txt_len);
	hOvey = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_AUTOCHECKBOX,
			200, 99, 195, 20, hWindow, (HMENU) IDC_OVEY, hInstance, NULL);

	SendMessage(hOvey, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	buttonwndproc = (WNDPROC)SetWindowLong(hOvey, GWL_WNDPROC, (LONG)ButtonProc);

	// Data read from card
	LoadString(hInstance, IDS_RAWDATA, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 122,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hRAW = CreateWindowEx(WS_EX_CLIENTEDGE, "STATIC", NULL,
			WS_CHILD | WS_VISIBLE,
			110, 121, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hRAW, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	// Read value filtered by data mask
	LoadString(hInstance, IDS_FINALVALUE, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 190, 122,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hFiltered = CreateWindowEx(WS_EX_CLIENTEDGE, "STATIC", NULL,
			WS_CHILD | WS_VISIBLE,
			300, 121, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hFiltered, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	// Explanation
	LoadString(hInstance, IDS_INST3, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE, 10, 148,
            370, 35, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));


	// Init mask
	LoadString(hInstance, IDS_INITMASK, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 185,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hInitMask = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			110, 184, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hInitMask, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hInitMask, EM_LIMITTEXT, 8, 0);
	SetWindowLong(hInitMask, GWL_WNDPROC, (LONG)EditProc);

	// Init value
	LoadString(hInstance, IDS_VALUE, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 190, 185,
            50, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hInitValue = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			250, 184, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hInitValue, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hInitValue, EM_LIMITTEXT , 8, 0);
	SetWindowLong(hInitValue, GWL_WNDPROC, (LONG)EditProc);

	//Output set button
	LoadString(hInstance, IDS_SETOE, txt, txt_len);
	hSetOE = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP,
			325, 184, 50, 20, hWindow, (HMENU) IDC_SETOE, hInstance, NULL);

	SendMessage(hSetOE, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hSetOE, GWL_WNDPROC, (LONG)ButtonProc);


	// ACK mask
	LoadString(hInstance, IDS_ACKMASK, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 207,
            100, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hACKmask = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			110, 206, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hACKmask, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hACKmask, EM_LIMITTEXT, 8, 0);
	SetWindowLong(hACKmask, GWL_WNDPROC, (LONG)EditProc);

	// ACK value
	LoadString(hInstance, IDS_VALUE, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 190, 207,
            50, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hACKvalue = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_UPPERCASE | WS_TABSTOP,
			250, 206, 75, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hACKvalue, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hACKvalue, EM_LIMITTEXT , 8, 0);
	SetWindowLong(hACKvalue, GWL_WNDPROC, (LONG)EditProc);

	//ACK button
	LoadString(hInstance, IDS_ACKNOW, txt, txt_len);
	hACK = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP,
			325, 206, 50, 20, hWindow, (HMENU) IDC_ACK, hInstance, NULL);

	SendMessage(hACK, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hACK, GWL_WNDPROC, (LONG)ButtonProc);

	// ACK Pulse width
	LoadString(hInstance, IDS_PULSE, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE | SS_RIGHT, 0, 230,
            135, 15, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));

	hPulse = CreateWindowEx(WS_EX_CLIENTEDGE, "Edit", NULL,
			WS_CHILD | WS_VISIBLE | ES_NUMBER | WS_TABSTOP | SS_RIGHT,
			145, 228, 40, 20, hWindow, NULL, hInstance, NULL);

	SendMessage(hPulse, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SendMessage(hPulse, EM_LIMITTEXT , 4, 0);
	SetWindowLong(hPulse, GWL_WNDPROC, (LONG)EditProc);



	// Checkbox - send always
	LoadString(hInstance, IDS_ACKALWAYS, txt, txt_len);
	hACKalways = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_AUTOCHECKBOX,
			200, 229, 195, 20, hWindow, (HMENU) IDC_ACKALWAYS, hInstance, NULL);

	SendMessage(hACKalways, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hACKalways, GWL_WNDPROC, (LONG)ButtonProc);


	// Explanation autodetection
	LoadString(hInstance, IDS_INST2, txt, txt_len);
	hLabel = CreateWindow("STATIC",
		txt,
			WS_CHILD | WS_VISIBLE, 10, 255,
            371, 35, hWindow, NULL, hInstance, NULL);

	SendMessage(hLabel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));


	// Autodetect button
	LoadString(hInstance, IDS_AUTOPRESS, txt, txt_len);
	hFindPress = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP,
			20, 290, 350, 25, hWindow, (HMENU) IDC_AUTO, hInstance, NULL);

	SendMessage(hFindPress, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hFindPress, GWL_WNDPROC, (LONG)ButtonProc);

	// Text viewer
	hText = CreateWindowEx(WS_EX_CLIENTEDGE, "STATIC",
		NULL,
			WS_CHILD | WS_VISIBLE, 20, 320,
            350, 50, hWindow, NULL, hInstance, NULL);

	SendMessage(hText, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));


	// OK Cancel buttons
	LoadString(hInstance, IDS_OK, txt, txt_len);
	hOk = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON,
			90, 380, 100, 28, hWindow, (HMENU) IDC_OK, hInstance, NULL);

	SendMessage(hOk, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hOk, GWL_WNDPROC, (LONG)ButtonProc);

	LoadString(hInstance, IDS_CANCEL, txt, txt_len);
	hCancel = CreateWindow("Button", txt,
			WS_CHILD | WS_VISIBLE | WS_TABSTOP,
			200, 380, 100, 28, hWindow, (HMENU) IDC_CANCEL, hInstance, NULL);

	SendMessage(hCancel, WM_SETFONT, (unsigned int)GetStockObject(DEFAULT_GUI_FONT),MAKELPARAM(TRUE, 0));
	SetWindowLong(hCancel, GWL_WNDPROC, (LONG)ButtonProc);

	// Make the Window visible
	ShowWindow(hWindow, SW_SHOWNORMAL);
	SetFocus(hCards);

	// Load cards detected on card combo box
	for(i = 0; i < numcards; i++)
	{
		sprintf(p, "%s [%u]", cards[i].description, cards[i].dwDeviceIndex);
		SendMessage(hCards, CB_ADDSTRING, 0, (LPARAM) p);
		if((cards[i].dwDeviceID == DeviceId) &&
			(cards[i].dwDeviceIndex == index))
			SendMessage(hCards, CB_SETCURSEL, (WPARAM) i, 0);
	}

	// Load current configuration values to the configuration window
	sprintf(p, "%u", Interval);
	SetWindowText(hInterval, p);
	sprintf(p, "%08X", flagmask);
	SetWindowText(hPress, p);
	sprintf(p, "%08X", flagvalue);
	SetWindowText(hValue, p);
	sprintf(p, "%08X", datamask);
	SetWindowText(hData, p);
	sprintf(p, "%08X", initmask);
	SetWindowText(hInitMask, p);
	sprintf(p, "%08X", initvalue);
	SetWindowText(hInitValue, p);
	sprintf(p, "%08X", ACKmask);
	SetWindowText(hACKmask, p);
	sprintf(p, "%08X", ACKvalue);
	SetWindowText(hACKvalue, p);
	sprintf(p, "%u", pulseLen);
	SetWindowText(hPulse, p);

	CheckDlgButton(hWindow, IDC_OVEY, noflag ? BST_UNCHECKED : BST_CHECKED);
	CheckDlgButton(hWindow, IDC_ACKALWAYS, ACKalways ? BST_CHECKED : BST_UNCHECKED);


	// Start GPIO updating thread
	if(DeviceId != 0)
	{
		status2 = 0;
		if(_beginthread(UpdateLoop, 0, hInstance) == -1) 
		{
			MessageBox(NULL, "Error creating update Thread", "Error", MB_ICONERROR);
			status2 = 2;
			return false;
		}
	}
	else
	{	// Warn the user the card was not detected
		LoadString(hInstance, IDS_NOCARD, txt, txt_len);
		MessageBox(hWindow, txt, "Error", MB_ICONERROR);
		EnableWindow(hFindPress, FALSE);
		EnableWindow(hACK, FALSE);
		EnableWindow(hSetOE, FALSE);
		status2 = 2;
	}

	return true;
}

// Polling thread
void UpdateLoop( LPVOID lpParameter )
{
	char buf[9];
	DWORD GPDATA, tmpmask;
	DWORD GPDATAprev;

	btCap.SetChipACK(0, 0, pulseLen);
	
	refresh = true;
	while(!status2)
	{
		// Poll for RC event
		GPDATA = btCap.GetGPDATA() & btCap.GetUsefulBits();
		if(refresh || (GPDATA != GPDATAprev))
		{
			GPDATAprev = GPDATA;
			refresh = false;
			// Show RAW GPDATA value
			sprintf(buf, "%08X", GPDATA);
			SetWindowText(hRAW, buf);
			// Read current data mask from textbox
			GetWindowText(hData, buf, 9);
			sscanf(buf, "%X", &tmpmask);
			// Show filtered GPIO value
			GPDATA &= tmpmask;
			sprintf(buf, "%08X", GPDATA);
			SetWindowText(hFiltered, buf);
		}
		Sleep(5);
	}
	// At this point the settings have been already updated
	btCap.SetChipInit(initmask, initvalue);
	btCap.SetChipACK(ACKmask, ACKvalue, pulseLen);
	status2 = 2;
	_endthread();
}

void Terminate()
{
	if(hWindow)
		DestroyWindow(hWindow);
	free(txt);
	if(classReg)
		UnregisterClass("BT8x8IR", hInstance);
}

// Configuration window events
void setoe_click(void)
{
	char p[9];
	DWORD mask, value;
	GetWindowText(hInitMask, p, 9);
	sscanf(p, "%X", &mask);
	GetWindowText(hInitValue, p, 9);
	sscanf(p, "%X", &value);
	btCap.SetChipInit(mask, value);
}

void ack_click(void)
{
	char p[9];
	DWORD mask, value, lpulseLen;
	GetWindowText(hACKmask, p, 9);
	sscanf(p, "%X", &mask);
	GetWindowText(hACKvalue, p, 9);
	sscanf(p, "%X", &value);
	GetWindowText(hPulse, p, 5);
	sscanf(p, "%lu", &lpulseLen);
	btCap.SetChipACK(mask, value, lpulseLen);
	btCap.ChipACK();
	btCap.SetChipACK(0, 0, pulseLen);
}

void cards_change(void)
{
	// Change the active card. Output settings will be reset
	// for safety
	int i = SendMessage(hCards, CB_GETCURSEL, 0, 0);
	if((cards[i].dwDeviceID == DeviceId) &&
		(cards[i].dwDeviceIndex == index))
		return;

	initmask = initvalue = 0;
	ACKmask = ACKvalue = 0;
	SetWindowText(hInitMask, "00000000");
	SetWindowText(hInitValue, "00000000");
	SetWindowText(hACKmask, "00000000");
	SetWindowText(hACKvalue, "00000000");
	btCap.SetChipInit(0, 0);
	btCap.SetChipACK(0, 0, pulseLen);

	// Enable selected card
	DeviceId = cards[i].dwDeviceID;
	index = cards[i].dwDeviceIndex;
	if(!btCap.UseCard(&(cards[i])))
	{
		LoadString(hInstance, IDS_ERRORCARD, txt, txt_len);
		MessageBox(NULL, txt, "Error", MB_ICONERROR);
	}
}

bool ok_click()
{
	char p[9];
	// Assign data to runtime variables
	GetWindowText(hInterval, p, 5);
	sscanf(p, "%u", &Interval);

	GetWindowText(hPress, p, 9);
	sscanf(p, "%X", &flagmask);
	GetWindowText(hValue, p, 9);
	sscanf(p, "%X", &flagvalue);
	GetWindowText(hData, p, 9);
	sscanf(p, "%X", &datamask);
	GetWindowText(hInitMask, p, 9);
	sscanf(p, "%X", &initmask);
	GetWindowText(hInitValue, p, 9);
	sscanf(p, "%X", &initvalue);
	GetWindowText(hACKmask, p, 9);
	sscanf(p, "%X", &ACKmask);
	GetWindowText(hACKvalue, p, 9);
	sscanf(p, "%X", &ACKvalue);
	GetWindowText(hPulse, p, 5);
	sscanf(p, "%u", &pulseLen);

	noflag = (IsDlgButtonChecked(hWindow, IDC_OVEY) != BST_CHECKED);
	ACKalways = (IsDlgButtonChecked(hWindow, IDC_ACKALWAYS) == BST_CHECKED);
	mask = datamask | flagmask;
	flagvalue &= flagmask;

	// Save data to config file
	return save_config();
}


void auto_click(void)
{
	if(status3)
	{
		// Start the thread to detect press mask and value
		status3 = 0;
		if(_beginthread(AutoTh, 0, hInstance) == -1)
		{
			MessageBox(hWindow, "Error creating autodetection Thread", "Error", MB_ICONERROR);
			status3 = 2;
			return;
		}
	}
	else
	{
		// Stop the thread
		status3 = 1;
	}
} 


// Autodetection thread
void AutoTh( LPVOID lpParameter )
{
	DWORD GPIOprev, GPIO;

	DWORD mask, ACKmask, value, lpulseLen;
	bool ACKalways, wasenabled;

	char cMask[50];
	unsigned c, i, lim = 100;
	DWORD *list;

	FILE *f;

	// Disable other buttons
	EnableWindow(hACK,    FALSE);
	EnableWindow(hOk,     FALSE);
	EnableWindow(hCancel, FALSE);

	LoadString(hInstance, IDS_END, txt, txt_len);
	SetWindowText(hFindPress, txt);

	// Set chip ACK temporarily and pause main thread
	wasenabled = enabled;
	enabled = false;
	GetWindowText(hACKmask, cMask, 9);
	sscanf(cMask, "%X", &ACKmask);
	GetWindowText(hACKvalue, cMask, 9);
	sscanf(cMask, "%X", &value);
	GetWindowText(hPulse, cMask, 5);
	sscanf(cMask, "%lu", &lpulseLen);
	btCap.SetChipACK(ACKmask, value, lpulseLen);
	ACKalways = (IsDlgButtonChecked(hWindow, IDC_ACKALWAYS) == BST_CHECKED);

	// Open log file
	f = fopen("Rc800.log", "wt");

	// Step 1: check for changing bits and mask them
	// Get no button status

	LoadString(hInstance, IDS_WAIT, txt, txt_len);
	SetWindowText(hText, txt);
	value = mask = btCap.GetUsefulBits();
	if(f != NULL) fprintf(f, "Useful bits: %X\n", mask);

	GPIOprev = btCap.GetGPDATA() & mask;
	for(i = 512; i && !status3; i--)
	{
		value &= ~(GPIOprev ^ (btCap.GetGPDATA() & mask));
		Sleep(1);
	}
	mask &= value;
	GPIOprev &= mask;

	if(f != NULL) fprintf(f, "Initial mask/value: %X/%X\n", mask, GPIOprev);

	list = (DWORD*) malloc(sizeof(DWORD) * lim);
	list[0] = GPIOprev;  // Status when no button is pressed

	LoadString(hInstance, IDS_PUSH, txt, txt_len);
	SetWindowText(hText, txt);

	// Step 2: build a list with all codes read

	c = 1;
	while(!status3)
	{
		GPIO = btCap.GetGPDATA() & mask;
		if(ACKmask && ACKalways)
			btCap.ChipACK();

		if(GPIO != GPIOprev)
		{
			if(ACKmask && !ACKalways)
				btCap.ChipACK();

			// Search code in list: if found, do not add again
			for(i = 0; i < c; i++)
				if(list[i] == GPIO) break;

			if(i == c)  // Not found, add
			{
				list[c++] = GPIO;
				LoadString(hInstance, IDS_CODES, txt, txt_len);
				sprintf(cMask, txt, c);
				SetWindowText(hText, cMask);
				if(c >= lim)
					list = (DWORD*)realloc(list, sizeof(DWORD) * (lim *= 2));
				if(f != NULL) fprintf(f, "%02u: %ld %X\n", c, clock(), GPIO);
			}
		}
		Sleep(5);
	}

	// Disable chip ACK and resume main thread
	btCap.SetChipACK(0, 0, pulseLen);
	enabled = wasenabled;

	if(DataAnalysis(list, c, f))
		LoadString(hInstance, IDS_SUCCESS, txt, txt_len);
	else
		LoadString(hInstance, IDS_FAIL, txt, txt_len);
	fclose(f);
	SetWindowText(hText, txt);
	// Enable other buttons
	EnableWindow(hACK,    TRUE);
	EnableWindow(hOk,     TRUE);
	EnableWindow(hCancel, TRUE);

	LoadString(hInstance, IDS_AUTOPRESS, txt, txt_len);
	SetWindowText(hFindPress, txt);
	status3 = 2;
	_endthread();
}

// Tries to guess Press and Data mask from read values
bool DataAnalysis(DWORD list[], unsigned c, FILE *f)
{
	DWORD dmask, pmask, vmask, tmpmask;
	bool setpress;

	unsigned i;
	char cMask[9];
	bool result;
	
	// Analysis start

	switch(c)
	{
	case 0:	// error
	case 1:	// no buttons pressed
		result = false;
		break;

	case 2:	// 1 button remote
		dmask = 0;
		pmask = list[0] ^ list[1];
		vmask = list[1] & pmask;
		setpress = true;
		result = true;
		break;

	default:	// 2 or more buttons, guess
		// First attempt to guess press mask
		// Search a bit not changed except in first code
		dmask = 0;
		for(i = 2; i < c; i++)
		{
			dmask |= (list[i-1] ^ list[i]);
		}
		// If that bit is not set in first code (no key status)
		// that is the press mask
		pmask = (~dmask & (list[0] ^ list[1]));
		if(pmask)
		{	// Successful
			vmask = list[1] & pmask;
			if(f != NULL) fprintf(f, "Type 1, data mask before fix: %X\n", dmask);
			dmask = FixRedundancy(dmask, list, c);
			setpress = true;
			break;
		}

		// If not successful, second attempt to guess press mask
		// Look for pairs with 1 bit changed, if more than 50% found,
		// take that bit as press mask
		unsigned start = 2;
		setpress = false;
		// If not successful Keep trying just in case the first
		// bit difference found was not the press mask
		// Starts searching at the next position
		while(!setpress && (start < c))
		{
			pmask = vmask = 0;
			for(i = start; i < c; i++)
			{
				tmpmask = (list[i-1] ^ list[i]);
				if(!(tmpmask & (tmpmask - 1))) // if only 1 bit is active
				{
					if(pmask)
					{
						// Counts number of times the 1 bit difference was found
						if(pmask == tmpmask) vmask++;
					}
					else
					{
						// First 1 bit difference found
						pmask = tmpmask;
						start = i + 1;
					}
				}
			}

			if((vmask * 4) >= c) // If more than 50% pairs had that 1 bit difference
			{
				setpress = true;
				vmask = ~list[0] & pmask;
				dmask &= ~pmask;
				if(f != NULL) fprintf(f, "Type 2, data mask before fix: %X\n", dmask);
			}
		}

		// If setpress is false, the Mask could not be guessed
		// Data mask is already stored on dmask.
		// To fix redundancies: if pmask is known, filter values
		if(setpress)
		{	// Create a new list with codes that have press bit set
			DWORD *list2;
			unsigned d = 0;

			list2 = (DWORD*) malloc(sizeof(DWORD) * c);
			for(i = 1; i < c; i++)
				if((list[i] & pmask) == vmask)
					list2[d++] = list[i];
			dmask = FixRedundancy(dmask, list2, d);
		}
		else
		{
			pmask = vmask = 0;
			dmask = FixRedundancy(dmask, list, c);
		}
		result = true;
	}

	// Analysis end

	if(result)
	{
		// Set guessed values to textboxes
		refresh = true;
		sprintf(cMask, "%08X", dmask);
		SetWindowText(hData, cMask);
		sprintf(cMask, "%08X", pmask);
		SetWindowText(hPress, cMask);
		sprintf(cMask, "%08X", vmask);
		SetWindowText(hValue, cMask);
		CheckDlgButton(hWindow, IDC_OVEY, setpress ? BST_CHECKED : BST_UNCHECKED);
		if(f != NULL) fprintf(f, "Press mask %X, Value %X, Data mask %X\n", pmask, vmask, dmask);
	}
	return result;
}

DWORD FixRedundancy(DWORD mask, DWORD list[], unsigned c)
{
	DWORD bit, newmask;
	unsigned i, j, rep;
	for(bit = 1; bit; bit <<= 1)
		if(mask & bit)
		{
			// Try to eliminate the bit and check for repeated codes
			newmask = mask & ~bit;
			rep = 0;
			for(i = 0; i < c; i++)
				for(j = i + 1; j < c; j++)
					if((list[i] & newmask) == (list[j] & newmask))
						if(rep++ == 1)
						{
							i = c;
							break;
						}
			//If there is one repetition and remote has more than 16 buttons
			//the bit is likely redundant
			if((!rep) || ((c >= 16) && (rep == 1)))
				mask = newmask;
		}
	return mask;
}